# This copy is forked and copied from here  https://github.com/ajoer/Wikipedia
# which is forked from here https://github.com/goldsmith/Wikipedia
from .wikipedia import *
from .exceptions import *

__version__ = (1, 4, 0)
